/*
 *     Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 *     Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; version 2.1 of the License.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package edu.ucsc.its.temerity.core

import edu.ucsc.its.temerity.TemClientConfig
import edu.ucsc.its.temerity.di.libModule
import edu.ucsc.its.temerity.di.loggerModule
import edu.ucsc.its.temerity.platformModule
import org.koin.core.Koin
import org.koin.dsl.koinApplication
import kotlin.time.Duration
import kotlin.time.Duration.Companion.minutes

/**
 * TemerityLibrary is the main entry point for the Temerity library.
 *
 * This object initializes the Koin dependency injection framework and returns a library or builder instance
 *
 */
public object TemerityLibrary {
  public val DEFAULT_WEB_TIMEOUT: Duration = 2.minutes

  private val koinApp = koinApplication {
    modules(
      loggerModule(),
      platformModule(),
      libModule,
    )
  }
  internal val koin: Koin = koinApp.koin
}

/**
 * Create a Temerity instance using Kotlin-DSL.
 * Adapted from the tmdb-api project: https://github.com/MoviebaseApp/tmdb-kotlin/raw/refs/heads/main/tmdb-api/src/commonMain/kotlin/app/moviebase/tmdb/Tmdb4.kt
 */
@TemDsl
public fun Temerity(block: TemClientConfig.() -> Unit): Temerity {
  val config = TemClientConfig().apply(block)
  return Temerity(config)
}
