/*
 *     Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 *     Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; version 2.1 of the License.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package edu.ucsc.its.temerity.test

import co.touchlab.kermit.Logger
import edu.ucsc.its.temerity.core.Temerity
import edu.ucsc.its.temerity.di.loggerModule
import io.kotest.core.extensions.Extension
import io.kotest.core.spec.style.FunSpec
import kotlinx.datetime.Clock
import kotlinx.datetime.TimeZone
import kotlinx.datetime.todayIn
import org.dotenv.vault.dotenvVault
import org.koin.core.parameter.parametersOf
import org.koin.test.KoinTest
import org.koin.test.inject
import kotlin.time.Duration.Companion.minutes

class DevUtilityTests :
  FunSpec(),
  KoinTest {

  override fun extensions(): List<Extension> = listOf(koinExtension(loggerModule()))
  private val kermit: Logger by inject { parametersOf("DevUtilityTests") }

  init {
    coroutineDebugProbes = true

    val dotenv = dotenvVault()

    lateinit var testTemerity: Temerity
    val today = Clock.System.todayIn(TimeZone.currentSystemDefault())

    beforeTest {
      testTemerity = Temerity {
        configureDevEnvironment(dotenv)
      }
    }

    timeout = 5.minutes.inWholeMilliseconds

    test("Delete all Canvas Test Student Users") {
      val testStudentList = testTemerity.getUsers().filter {
        it.firstName.startsWith("test", ignoreCase = true) &&
          it.lastName.startsWith("student", ignoreCase = true) &&
          it.emailAddress.isEmpty()
      }
      testStudentList.forEach {
        testTemerity.deleteUser(it.userId)
      }
    }
  }
}
